#!/usr/bin/perl 
#
# recode_gen.pl : recodes alphabeticly coded SNP systems into  numerical alleles. 

#-------------------------- help message begins -----------------
$name="recode_gen.pl";


$help_message="

HEADER:
$name 

NAME:
$name - A program for recoding alphabeticly coded systems into  alleles.

SYNOPSIS:
  perl $name [-help] [-d INPUT DATA FILE] [-r OUTPUT DATA FILE] [-s SEPARATOR]  


OPTIONS:
-help runs help for the command

-d    INPUT DATA FILE - the name of file with raw genotype data, SNP.dat  by default

-r    OUTPUT DATA FILE - the name of file to store re-coded genotype date, recSNP.dat  by default

-s    SEPARATOR specifies a separator between entries for the input_data_file. 
      A \";\" is assumed by default, but one can specify any character (e.g. \"-s ZZxZ\")

None, one, two, or all command line options may be used. Order of the options is irrelevant.

Detailed documentation: 
This program was written to recode alphabeticly coded SNP systems into  numerical alleles. 

Input: 

INPUT DATA FILE is a free format data file. This is text file,
where entries are separated by semicolon (by default).
One column is required with name code. It contains a uniquie person identification number.
The others columns contain genotype data.
The first line is a header line, wich contains names of columns.
In this file before a header line, there might be comment lines, starting with \"#\".
Untyped people must have a '0' or ' ' genotype. Alphabetic systems must be coded without a space between the alleles.    


Output: 

OUTPUT DATA FILE - a recoded  file, using the numerical alleles as genotype codes. 

Example: 

Input: 
SNP.dat: 

# marker1:m1, marker2:m2,marker3:m3
code;m1;m2;m3
3244;AA;GT;AC
3345;CC;TT;AA
4780;AC;GG;CC

Output:
Recoded file: recSNP.dat

code;m1;m1;m2;m2;m3;m3
3244;1;1;1;2;1;2
3345;2;2;2;2;1;1
4780;1;2;1;1;2;2

RESOURCES:

DIAGNOSTICS:

SEE ALSO:


COPYRIGHT:
recode_gen.pl is Copyright (c) 2004 Erasmus MC Rotterdam and ICG Novosibirsk

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 2 of the License, or (at your opinion) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

BUGS:

AUTHOR:
Irina Zorkoltseva 


CREATED:
22.11.2004 14:30

LAST MODIFIED:
22.11.2005 12:00

LANGUAGE:
Perl v5.8.4

OS: 
XXX Linux vXX.XX
";
#-------------------------- help message ends -----------------

#******** default parametrs ************************


# INPUT
$indat="SNP.dat"; #input file with genotype data
#OUTPUT
$outdat="recSNP.dat";  # output genotype data file
$err="recode_gen.err";   # output file with errors diagnostic    
$sep=$out=";";    # input and output collumn separator
$cod="cod.rec";

#**************** get command line arguments 
#die "run recode_ped.pl -help for details\n" if (@ARGV<1);

if ($ARGV[0] eq "-help") {
	print "${help_message}";
	open (HELP, ">recode_gen.help");
	print (HELP "${help_message}");
	die ""; 
}; 

#*************** reparametrization *************************
	$narg = @ARGV;
	for ($i=0;$i<$narg-1;$i++) 
	{
		$sep=$ARGV[$i+1] if ($ARGV[$i] =~ /-s/);
		$indat=$ARGV[$i+1] if ($ARGV[$i] =~ /-d/); # input data file
		$outdat=$ARGV[$i+1] if ($ARGV[$i] =~ /-r/);  # rezult file
	}

	&PRN_PAR;


open (ERR,">$err"); # open file for errors
${error_m}="";


#************open and read the genotype data******************
         
open(INP,"$indat") or die "FATAL ERROR0: cannot open file with genotype data $indat: $!\n"; 

#*********check the header line ********************

$hea="# ";
while ($hea =~ /^#/) {
	$line++;
	$hea = <INP>;
}
chomp $hea;                                       
@header = split(/$sep/, $hea, -1 );
#print @header;
$encod=0;
$no_err=0;

#**************count the number of required entries
                                                
for ($i=0;$i<@header;$i++) {
	$encod++ if ($header[$i] =~ /^code$/);

}
$error_m="${error_m}ERROR1: Rsp code (code) column not found in input file\n" if ($encod==0);
$error_m="${error_m}ERROR1: Rsp code (code) column found more then once in input file\n" if ($encod>1);

## die "ERROR: Rsp code (rsp_cod) column not found in input file\n" if ($encod==0);
## die "ERROR: Rsp code (rsp_cod) column found more then once in input file\n" if ($encod>1);

for ($i=0;$i<@header;$i++) {
	$encod=$i if ($header[$i] =~ /^code$/); 
}
#header of marker names

$head_trait="";
$n_tr=0;
$h_tr=1;

for ($i=0;$i<@header;$i++)
 {	
next if $i eq $encod;
	$head_trait[$n_tr]=$header[$i];
	$n_tr++; 
}

$error_m="${error_m}ERROR1: marker's column not found in input file\n" unless($n_tr);



######################## check duplicated marker...

for ($i=0; $i<$n_tr; $i++) {
             $tt=$head_trait[$i];
	for ($j=$i+1; $j<$n_tr; $j++) {
	$error_m="${error_m}ERROR1: Marker $tt found more then once in input file\n" if ($tt eq $head_trait[$j]);
}

}



###################read in the genotype file ...

#print "Reading genotype data...\n";

$noent = @header;
$nids=0;
$i=0;

while (<INP>) {
	next if (/^#/);
	next if (/^ *$/);
	chomp;
	$_="$_${sep}";
	@temp = split/$sep/;
	$tmp=@temp-1;
	$rsp[$i]=$temp[$encod];

	for ($j=0;$j<$n_tr;$j++)
 	{
		$tr[$i][$j]=$temp[$j+1];
		
#print "$tr[$i][$j]\n";

	}                                                              

 	$i++;
	splice @temp,0,$tmp;
}

$nids=$i;
$error_m="${error_m}ERROR: empty genotype data file\n"  unless ($nids);

######################## check duplicated code...

for ($i=0; $i<$nids; $i++) {
             $cid=$rsp[$i];
	for ($j=$i+1; $j<$nids; $j++) {
	$error_m="${error_m}ERROR3: Personal code $cid duplicated in lines $i and $j\n" if ($cid eq $rsp[$j]);
}

}

close INP;
#print "End reading genotype file\n";

##**************** check trait *************

$n_err=0;
for ($j=0; $j<$n_tr; $j++) {

     $dif_type=0;
     $dif_al[$j]=0;	
    for ($i=0; $i<$nids; $i++) {

        next if ($tr[$i][$j] eq ~/^ \s/ );	
        next if ($tr[$i][$j] eq "" );	
        next if ($tr[$i][$j] eq " " );	

	   @allel=split//,$tr[$i][$j];
##print "locus $j ind $i: @allel\n";
    for ($k=0; $k<2; $k++) 
  {

            if ($dif_al[$j]) {

		$yes=0;
		for ($l=0; $l<$dif_al[$j]; $l ++) 
		{
		$yes++ if ($allel[$k] eq $Type_al[$l][$j])
		}
		unless ($yes) { 
			$Type_al[$dif_al[$j]][$j] = $allel[$k];
		        $dif_al[$j]++;
		              }

	                   }
	else {	
		$Type_al[$dif_al[$j]][$j] = $allel[$k];
		$dif_al[$j] ++;	

	      }			
	                   
	 
  }  # end k
	 
  } # end i
#print "marker $head_trait[$j]: number of different alleles is equel $dif_al[$j]:1=$Type_al[0][$j] 2=$Type_al[1][$j]3=$Type_al[2][$j]4=$Type_al[3][$j]end\n";
#print "Genotypes: @Type\n";
   

    }      # end j 

if($error_m) {
   print (ERR "$error_m");
   die "${error_m}\nProgram has found fatal errors in the input data.
After correcting the input files rerun program.\n";
}              

open (COD,">$cod"); # open file for allele's codes
for ($j=0; $j<$n_tr; $j++) {
print (COD "Locus $head_trait[$j]:\n");
for ($l=1; $l<$dif_al[$j]+1; $l ++){
print (COD "$l  $Type_al[$l-1][$j]\n"); }}
 

##die "ERRORS exist. Check the value of trait\n" if ($error_m);

#*************** recodes genotype *************

open(OUT,">$outdat") or die "ERROR0: cannot open out genotype file $outdat: $!\n"; 

$head="code";
    for ($j=0; $j<$n_tr; $j++) {
$head="${head}${out}$head_trait[$j]${out}$head_trait[$j]";}
print(OUT "${head}\n");

 for ($i=0; $i<$nids-1; $i++) {
 $temp="$rsp[$i]";
    for ($j=0; $j<$n_tr; $j++) {

	if ($tr[$i][$j]!=~/^ \s+/ ){
	   @allel=split//,$tr[$i][$j];
##print "$i, $j,$tr[$i][$j], @allel\n";
	$a1=0;
	$a2=0;

		for ($l=0; $l<$dif_al[$j]; $l ++) 
		{
	$a1=$l+1 if($allel[0] eq $Type_al[$l][$j]);
	$a2=$l+1 if($allel[1] eq $Type_al[$l][$j]);
		}
           $temp="${temp}${out}$a1${out}$a2";
  	}
        else {
          $temp="${temp}${out}0${out}0" ;	
	}
}
  print(OUT "${temp}\n");
}
  print "END OF PROGRAM: recoded genotype file is created\n";

sub PRN_PAR {
print "\n$name input patameters
\tField input_separator: -s=\"$sep\"
\tField output_separator: -o=\"$out\"
\tInput genotype file: -d=\"$indat\"
\tOutput genotype file: -r=\"$outdat\"\n\n"
} 
